<?php

/**
 * @package SimplePortal
 *
 * @author SimplePortal Team
 * @copyright 2015 SimplePortal Team
 * @license BSD 3-clause
 * @version 1.0.0 Beta 2
 */

if (!defined('ELK'))
{
	die('No access...');
}

/**
 * Image Attachment Block, Displays a list of recent post image attachments
 */
class Photo_Month_Block extends SP_Abstract_Block
{
	/**
	 * Constructor, used to define block parameters
	 *
	 * @param Database|null $db
	 */
	public function __construct($db = null)
	{
		global $txt;

		// Select Options
		$txt['sp_param_Photo_Month_watermark_options'] = 'Top_Left|Top_Right|Bottom_Right|Bottom_Left';
		$txt['sp_param_Photo_Month_scale_options'] = 'Cover|Contain';
		$txt['sp_param_Photo_Month_ratio_options'] = '4by3|16by9|2by1|3by1';

		// Block Options
		$txt['sp_param_Photo_Month_photo'] = 'Photo ID';
		$txt['sp_param_Photo_Month_watermark'] = 'Watermark Location';
		$txt['sp_param_Photo_Month_scale'] = 'Scale';
		$txt['sp_param_Photo_Month_ratio'] = 'Photo Ratio';
		$txt['sp_param_Photo_Month_caption'] = 'Caption';

		$this->block_parameters = array(
			'space1' => 'space',
			'photo' => 'int',
			'watermark' => 'select',
			'caption' => 'text',
			'scale' => 'select',
			'ratio' => 'select',
			'space2' => 'space',
		);

		parent::__construct($db);
	}

	/**
	 * Returns the name of the block
	 */
	public static function blockName()
	{
		global $txt;

		$txt['sp_function_Photo_Month_label'] = 'Photo of the Month';
	}

	/**
	 * Returns the description of the block
	 */
	public static function blockDescription()
	{
		global $txt;

		$txt['sp_function_Photo_Month_desc'] = 'Allows selection of a gallery image for display in a block.';
	}

	/**
	 * Initializes a block for use.
	 *
	 * - Called from portal.subs as part of the sportal_load_blocks process
	 *
	 * @param mixed[] $parameters
	 *        'photo' => Gallery id of the image
	 *        'watermark' => Where to show the watermark overlay
	 *        'caption' => Text for below the image
	 *        'scale' => how to scale the image
	 *        'ratio' => basic image ratio
	 * @param int $id
	 */
	public function setup($parameters, $id)
	{
		global $txt;

		$watermark = array('top_left', 'top_right', 'bottom_right', 'bottom_left');
		$scale = array('cover', 'contain');
		$ratio = array('4by3', '16by9', '2by1', '3by1');
		$height = array(500, 250, 400, 200);

		$this->data['photo'] = empty($parameters['photo']) ? 0 : (int) $parameters['photo'];
		$this->data['watermark'] = empty($parameters['watermark']) ? 'top_left' : $watermark[$parameters['watermark']];
		$this->data['caption'] = empty($parameters['caption']) ? '' : (string) $parameters['caption'];
		$this->data['scale'] = empty($parameters['scale']) ? 'cover' :  $scale[$parameters['scale']];
		$this->data['ratio'] = empty($parameters['ratio']) ? '4by3' :  $ratio[$parameters['ratio']];
		$this->data['height'] = empty($parameters['ratio']) ? 600 :  $height[$parameters['ratio']];

		// No attachments, at least none that they can see
		if (empty($this->data['photo']))
		{
			$this->data['error_msg'] = $txt['error_sp_no_attachments_found'];
			$this->setTemplate('template_sp_potm_error');

			return;
		}

		$this->setTemplate('template_sp_potm');
	}
}

/**
 * Error template for this block
 *
 * @param mixed[] $data
 */
function template_sp_potm_error($data)
{
	echo $data['error_msg'];
}

/**
 * Main template for this block
 *
 * @param mixed[] $data
 */
function template_sp_potm($data)
{
	// Build the output for display
	echo '
		<style> 
		.potm {
			overflow: hidden;
			position: relative;
			display: block;
		}
		.potm:before {
			content: "";
			display: block;
			width: 100%;
		}
		.potm_3by1:before {
			padding-bottom: 33.33333%;
		}
		.potm_2by1:before {
			padding-bottom: 50%;
		}
		.potm_16by9:before {
			padding-bottom: 56.25%;
		}
		.potm_4by3:before {
			padding-bottom: 75%;
		}
		.potm_contain {
			background-size: contain;
		}
		.potm_cover {
			background-size: cover;
		}
		.potm_fleximage {
			margin: 0 auto;
			background-image: url(http://www.lazydazeowners.com/MGalleryItem.php?id=', $data['photo'], ';preview);
			background-repeat: no-repeat;
			background-position: 50%;
			max-height: ', $data['height'], 'px;
			max-width: 1200px;
		}
		figcaption {
			padding: 5px;
			font-size: 1em;
			font-weight: 600;
			border: none;
			background: transparent;
			word-wrap:normal;
			text-align: center;
		}
		.potm_watermark {
			position: absolute;
			opacity: 0.55;
			font-size: 1.5em;
			width: 100%;
			z-index: 2;
			font-weight: 600;
			color: #fff;
			text-shadow: -1px -1px 0 #000, 1px -1px 0 #000, -1px 1px 0 #000, 1px 1px 0 #000;
		}
		.potm_top_left {
			top: .5em;
			text-align: left;
			padding: 0 0 0 10px;
		}
		.potm_top_right {
			top: .5em;
			text-align: right;
			width: 99%;
		}
		.potm_bottom_right {
			bottom: .5em;
			text-align: right;
			width: 99%;
		}
		.potm_bottom_left {
			bottom: .5em;
			text-align: left;
			padding: 0 0 0 10px;
		}
		</style>
		<a href="http://www.lazydazeowners.com/MGalleryItem.php?id=', $data['photo'], '" rel="gallery" class="fancybox">
			<div class="potm_fleximage potm potm_', $data['ratio'], ' potm_', $data['scale'], '">
				<span class="potm_watermark potm_', $data['watermark'], '">Lazy Daze Owners\' Group</span>
			</div>
			<figcaption>', $data['caption'], '</figcaption>
		</a>';
}
