<?php

/**
 * @package SimplePortal
 *
 * @author SimplePortal Team
 * @copyright 2015 SimplePortal Team
 * @license BSD 3-clause
 * @version 1.0.0 Beta 2
 */

if (!defined('ELK'))
{
	die('No access...');
}

/**
 * Board Block, Displays a list of posts from selected board(s)
 *
 * @param mixed[] $parameters
 * 			'board' => Board(s) to select posts from
 * 			'limit' => max number of posts to show
 * 			'start' => id of post to start from
 *			'length' => preview length of the post
 *			'avatar' => show the poster avatar
 *			'per_page' => number of posts per page to show
 *			'attachment' => Show the first attachment as "blog" image
 * @param int $id - not used in this block
 * @param boolean $return_parameters if true returns the configuration options for the block
 */
class Unread_Posts_Block extends SP_Abstract_Block
{
	/**
	 * Constructor, used to define block parameters
	 *
	 * @param Database|null $db
	 */
	public function __construct($db = null)
	{
		$this->block_parameters = array(
			'boards' => 'boards',
			'limit' => 'int',
			'type' => 'select',
			'display' => 'select',
			'per_page' => 'int',
		);

		$this->refresh = 60;

		parent::__construct($db);
	}

	/**
	 * Initializes a block for use.
	 *
	 * - Called from portal.subs as part of the sportal_load_blocks process
	 *
	 * @param mixed[] $parameters
	 * @param int $id
	 */
	function setup($parameters, $id)
	{
		global $scripturl, $txt, $settings, $modSettings, $context;

		// Break out / sanitize all the parameters
		// Clean the parameters for this block
		$boards = !empty($parameters['boards']) ? explode('|', $parameters['boards']) : null;
		$limit = !empty($parameters['limit']) ? (int) $parameters['limit'] : 5;
		$type = 'ssi_recent' . (empty($parameters['type']) ? 'Posts' : 'Topics');
		$this->data['display'] = empty($parameters['display']) ? 'compact' : 'full';
		$per_page = !empty($parameters['per_page']) ? (int) $parameters['per_page'] : 0;

	}
}

/**
 * Error template for this block
 *
 * @param mixed[] $data
 */
function template_sp_unread_error($data)
{
	echo $data['error_msg'];
}

/**
 * Main template for this block
 *
 * @param mixed[] $data
 */
function template_sp_unread($data)
{
	global $context, $scripturl, $txt;

	// Auto video embedding enabled?
	if ($data['embed_videos'])
	{
		addInlineJavascript('
		$(document).ready(function() {
			$().linkifyvideo(oEmbedtext);
		});', true);
	}

	// Output all the details we have found
	foreach ($data['news'] as $news)
	{
		$attachment = $news['attachment'];

		echo '
			<h3 class="category_header">
				<span class="floatleft sp_article_icon">', $news['icon'], '</span><a href="', $news['href'], '" >', $news['subject'], '</a>
			</h3>
			<div id="msg_', $news['message_id'], '" class="sp_article_content">
				<div class="sp_content_padding">';

		// @todo replace the <img> with $news['avatar']['img'] and some css for the max-width
		if (!empty($news['avatar']['href']))
		{
			echo '
					<a href="', $scripturl, '?action=profile;u=', $news['poster']['id'], '">
						<img src="', $news['avatar']['href'], '" alt="', $news['poster']['name'], '" style="max-width:40px" class="floatright" />
					</a>
					<div class="middletext">', $news['time'], ' ', $txt['by'], ' ', $news['poster']['link'], '<br />', $txt['sp-articlesViews'], ': ', $news['views'], ' | ', $txt['sp-articlesComments'], ': ', $news['replies'], '</div>';
		}
		else
		{
			echo '
					<div class="middletext">', $news['time'], ' ', $txt['by'], ' ', $news['poster']['link'], ' | ', $txt['sp-articlesViews'], ': ', $news['views'], ' | ', $txt['sp-articlesComments'], ': ', $news['replies'], '</div>';
		}

		echo '
					<div class="post"><hr />';

		if (!empty($attachment))
		{
			echo '
						<div class="sp_attachment_thumb">';

			// If you want Fancybox to tag this, remove nfb_ from the id
			echo '
							<a href="', $news['href'], '" id="nfb_link_', $attachment['id'], '"><img src="', $attachment['href'], '" alt="" title="', $attachment['name'], '" id="thumb_', $attachment['id'], '" /></a>';

			echo '
						</div>';
		}

		echo $news['body'], '
					</div>
				<div class="righttext">', $news['link'], ' ', $news['new_comment'], '</div>
			</div>
		</div>';
	}

	// Pagination is a good thing
	if (!empty($context['sp_boardNews_page_index']))
	{
		echo '
		<div class="sp_page_index">',
			template_pagesection(false, '', array('page_index' => 'sp_boardNews_page_index')), '
		</div>';
	}
}